### Section 4.1 (Parametric model)
##################################

library(survival)
sur24<-with(tooth24,Surv(left,right,type="interval2"))
dim(sur24)
sur24[1:5]

tooth24$cens<-with(tooth24,ifelse(right==999,0,3))
sur24b<-with(tooth24,Surv(left,right,cens,type="interval"))
dim(sur24b)
sur24b[1:5]

weimod<-survreg(sur24~sex+dmf,data=tooth24,dist="weibull")
summary(weimod)

(new<-data.frame(sex=rep(0:1,2),dmf=rep(0:1,each=2)))
Median<-round(predict(weimod,newdata=new,type='quantile',p=0.5),2)+5
data.frame(new,Median)
pred<-predict(weimod,newdata=new,type='quantile',p=c(0:999/1000))

x11(width=10,height=6)
par(font=2,font.lab=2,font.axis=2,las=1,lwd=3,cex=1.5)
plot(pred[1,],c(0:999/1000),type="l",xlab = "Years from age 5",ylab = "Probability")
lines(pred[2,],c(0:999/1000),type="l",col=2)
lines(pred[3,],c(0:999/1000),type="l",lty=2)
lines(pred[4,],c(0:999/1000),type="l",col=2,lty=2)
legend("topleft",c('Boy: dmf=0','Girl: dmf=0','Boy: dmf=1','Girl: dmf=1'),
      text.width=strwidth('Boy: dmf=0 '),lty=c(1,1,2,2),col=c(1,2,1,2),lwd=3)


###########################
### Section 4.2 (Cox model)
###########################

# install.packages("int.cox")
library(intcox)

tooth24$rightNA<-with(tooth24,ifelse(right==999,NA,right))
coxmod<-intcox(Surv(left,rightNA,type="interval2")~sex+dmf,tooth24)
coxmod
summary(coxmod)


############################################################
### Bootstrap confidence intervals (code not shown in paper)
############################################################
intcox.boot <- function(i, form) {
   boot.sample <- with(tooth24,sample(id2, nrow(tooth24), replace = T))
   data.sample <- tooth24[boot.sample, ]
   boot.fit <- intcox(form, data = data.sample, no.warnings = TRUE)
   return(list(coef = coef(boot.fit), term = boot.fit$termination))
}

set.seed(246)
n.rep <- 999
tooth24.boot <- lapply(1:n.rep, intcox.boot, form = Surv(left, rightNA, type = "interval2") ~ sex + dmf)
tooth24.boot <- matrix(unlist(tooth24.boot), byrow = T, nrow = n.rep)
colnames(tooth24.boot) <- c(names(coef(coxmod)), "termination")
inf.level <- 0.05
sex.ord <- order(tooth24.boot[, "sex"])
dmf.ord <- order(tooth24.boot[, "dmf"])
pos.lower <- ceiling((n.rep + 1) * (inf.level/2))
pos.upper <- ceiling((n.rep + 1) * (1 - inf.level/2))
ci.sex <- tooth24.boot[sex.ord, "sex"][c(pos.lower, pos.upper)]
ci.dmf <- tooth24.boot[dmf.ord, "dmf"][c(pos.lower, pos.upper)]
ci.sex
ci.dmf

rm(tooth24.boot,sex.ord,dmf.ord,pos.lower,pos.upper,n.rep,inf.level)
